/*
 * NET Thermometer UDP client
 *
 *  Copyright (C) 1999-2001  Claudio Lanconelli
 *                                                                     
 *  e-mail: lancos@libero.it                                          
 *  http://www.LancOS.com                                         
 *
 * $Id: udpclient.cpp,v 1.7 2005/02/27 00:28:04 lanconel Exp $
 */

/*
 * MRTG Parameter Added By
 * Mike Roest
 * msroest@blahz.ab.ca
 */

//-------------------------------------------------------------------------//
//                                                                         //
// This program is free software; you can redistribute it and/or           //
// modify it under the terms of the GNU  General Public License            //
// as published by the Free Software Foundation; either version2 of        //
// the License, or (at your option) any later version.                     //
//                                                                         //
// This program is distributed in the hope that it will be useful,         //
// but WITHOUT ANY WARRANTY; without even the implied warranty of          //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       //
// General Public License for more details.                                //
//                                                                         //
// You should have received a copy of the GNU  General Public License      //
// along with this program (see COPYING);     if not, write to the         //
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. //
//                                                                         //
//-------------------------------------------------------------------------//

#ifdef	WIN32
#else
# ifdef	_LINUX_
# include <sys/types.h>
# include <sys/time.h>
# include <unistd.h>
# endif
#endif

#include <string.h>
#include <ctype.h>
#include <errno.h>

#include "../readtemp.h"
#include "inet.h"

#ifndef INADDR_NONE
#define INADDR_NONE	0xffffffff
#endif

#define DELAY_TIME	15

typedef void Sigfunc(int);

void getargs(int, char**, struct param*);
//void alarm_handler(int);
//Sigfunc* signal_intr(int, Sigfunc*);

struct param {
	char	help;
	int		port;
	char *	addr;
	int		mean;
	int		first;
	int		last;
	UnitType unit;
	int		digits;
  bool            mrtg;
};

void SocketEnd();
int SocketInit();
char *GetSockErrorString(int val);

static struct param	parameters = {
	0,
	SERV_PORT,
	SERV_ADDR,
	0,		//no mean
	1,		//first
	1,		//last
	uCELSIUS,
  1,		//digits
  0               //Set default to be no mrtg format.
};

int main(int argc, char *argv[])
{
	SOCKET				sockfd;
	struct sockaddr_in	cli_addr, serv_addr;
	struct hostent*		hostptr;
	unsigned int		inaddr;

	/*
	 * Check for the parameters
	 */
	getargs (argc, argv, &parameters);
  if(!parameters.mrtg)   //if not for MRTG output print COPYRIGHT
	printf("NetThermometer (Client) by LancOS 1999-2001 " VERSION "\n\n");

	if (parameters.help != 0)
	{
    printf(	"Usage: %s [-help] [-port port] [-addr address] [-mean] [-what F-L] [-mrtg] [-far] [-digits N]\n", argv[0]);
		printf(	"       -help show this help\n"
				"       -port service port number (%d)\n", SERV_PORT);
		printf(	"       -addr server address (localhost)\n"
				"       -mean perform mean between sensors\n"
				"       -what which sensor to retrieve (1-1)\n"
		"       -mrtg return reults in MRTG format\n"
				"       -digits how many decimal digits to show\n"
				"       -far  farenheit unit of measure instead of celsius\n");
	}
	else
	{
		/*
		 * Initialize Sockets
		 */
		if ( !SocketInit() )
		{
			fprintf(stderr, "error in Socket initialization\n");
			exit(1);
		}

		/*
		 * Fill in the "serv_addr" field with the server address
		 */
		bzero((char *)&serv_addr, sizeof(serv_addr));
		serv_addr.sin_family      = AF_INET;
		serv_addr.sin_port        = htons(parameters.port);
		if ((inaddr = inet_addr(parameters.addr)) != INADDR_NONE)
			serv_addr.sin_addr.s_addr = inaddr;
		else if ((hostptr = gethostbyname(parameters.addr)) != NULL)
			serv_addr.sin_addr = *((struct in_addr*)hostptr->h_addr);
		else
			myerror(1, "client: host name error");

		/*
		 * Open the UDP socket (Internet datagram socket).
		 */
		sockfd = socket(AF_INET, SOCK_DGRAM, 0);
		if ( sockfd == INVALID_SOCKET )
		{
#ifdef	WIN32
			errno = WSAGetLastError();
			fprintf(stderr, "%s\n", GetSockErrorString(errno));
#endif
			myerror(1, "client: can't open datagram socket");
		}

		/*
		 * Request for a local address
		 */
		bzero((char *)&cli_addr, sizeof(cli_addr));
		cli_addr.sin_family      = AF_INET;
		cli_addr.sin_addr.s_addr = htonl(INADDR_ANY);
		cli_addr.sin_port        = htons(0);
		if (bind(sockfd, (struct sockaddr *)&cli_addr, sizeof(cli_addr)) < 0)
			myerror(1, "client: can't bind local address");

		/*
		 * Send the service request to the server
		 */
		ThermoRequest request;
		int n = sizeof(request);
		strcpy(request.Id, THERMOID);
		request.Action = parameters.mean ? ACTION_MEAN : ACTION_GET;
		request.first = parameters.first;
		request.last = parameters.last;
		if (sendto(sockfd, (char *)&request, n, 0, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) != n)
			myerror(1, "client: sendto error on socket");

		fd_set rfds;
		struct timeval tv;

		/* Watch stdin (fd 0) to see when it has input. */
		FD_ZERO(&rfds);
		FD_SET(sockfd, &rfds);

		/* Wait up to 15 seconds. */
		tv.tv_sec = DELAY_TIME;
		tv.tv_usec = 0;

		n = select(sockfd+1, &rfds, NULL, NULL, &tv);
		/* Don't rely on the value of tv now! */
		if (n < 0)       //Error
		{
			myerror(1, "client: select() error");
		}
		else
		if (n == 0)  //Timeout
		{
			myerror(1, "client: no response from server (timeout)");
		}

		/*
		 * Wait for a reply from the server
		 */
		ThermoReply reply;
		socklen_t fromlen = sizeof(struct sockaddr);
		struct sockaddr fromaddr;
		n = recvfrom(sockfd, (char *)&reply, sizeof(reply), 0, &fromaddr, &fromlen);
		if (n < 0)
		{
#ifdef	WIN32
			errno = WSAGetLastError();
			fprintf(stderr, "%s\n", GetSockErrorString(errno));
#endif
			myerror(1, "client: recvfrom() error");
		}

		/*
		 * Print the server answer to stdout
		 */
		for (n = 0; n < N_THERM; n++)
			reply.vals[n] = ntohs(reply.vals[n]);
    if(!parameters.mrtg) //if not for MRTG output print Status from get temp
		printf("Result: %d *** %s\n\n", reply.result, reply.msg);

		/* The mean result is always stored in first value */
		if (parameters.mean)
		{
			request.first = 1;
			request.last = 1;
		}

		for (n = request.first-1; n < request.last; n++)
		{
      if(!parameters.mrtg)  //if not for MRTG output print Sensor Number
			printf("Sensor n.%d: ", n+1);

			if (reply.status[n])
			{
				printf("Error %d\n", reply.status[n]);
			}
			else
			{
	double temp = (parameters.unit == uCELSIUS) ? (double)reply.vals[n] / 256.0 : (double)reply.vals[n] * (9.0 / (256.0 * 5.0)) + 32.0;
	if(!parameters.mrtg) {  ///If not for MRTG output print out temp with requested digits
				switch (parameters.digits)
				{
				case 0:
					printf("%d", (int)temp);
					break;
				case 2:
					printf("%5.2f", temp);
					break;
				case 3:
					printf("%6.3f", temp);
					break;
				case 4:
					printf("%7.4f", temp);
					break;
				default:
					printf("%4.1f", temp);
					break;
				}
	}
	else {   //Else printout only int part as MRTG barfs on floats.
	  printf("%d\n", (int)temp); 
	}
	if(!parameters.mrtg)  //If not for MRTG prinout unit.
				printf(" %c\n", (parameters.unit == uCELSIUS) ? 'C' : 'F');
			}
		}
		closesocket(sockfd);

		SocketEnd();
	}
	return 0;
}

/*
 * Parse the command line and store parameters in a structure
 */
void getargs(int argc, char* argv[], struct param* parameters)
{
	int n = 1;
	int err = 0;

	while(n < argc)
	{
		if ( strcmp(argv[n], "-help") == 0 ||
			 strcmp(argv[n], "-h") == 0 ||
			 strcmp(argv[n], "-?") == 0 )
			parameters->help = 1;
		else
		if (strcmp(argv[n], "-port") == 0)
		{
			parameters->port = 0;
			if (++n < argc)
				parameters->port = atoi(argv[n]);

			if (parameters->port == 0)
				err = 1;
		}
		else
		if (strcmp(argv[n], "-addr") == 0)
		{
			if (++n < argc && argv[n][0] != '-')
				parameters->addr = argv[n];
			else
				err = 1;
		}
		else
		if (strcmp(argv[n], "-digits") == 0)
		{
			if ( ++n < argc && isdigit(argv[n][0]) )
				parameters->digits = atoi(argv[n]);
			else
				err = 1;
		}
		else
		if ( strcmp(argv[n], "-mean") == 0 )
		{
			parameters->mean = 1;
		}
		else
		if (strcmp(argv[n], "-far") == 0)
		{
			parameters->unit = uFARENHEIT;
		}
		else
		if (strcmp(argv[n], "-what") == 0)
		{
			parameters->first = 0;
			if (++n < argc)
			{
				sscanf(argv[n], "%d-%d", &(parameters->first), &(parameters->last));
				if (parameters->first < 1 || parameters->first > N_THERM)
					parameters->first = 1;
				if (parameters->last < 1 || parameters->last > N_THERM)
					parameters->last = 1;
				if (parameters->last < parameters->first)
					parameters->last = parameters->first;
			}

			if (parameters->first == 0)
				err = 1;
		}
	        else  //Set mrtg to true in parameters if -mrtg was on command line
		  if (strcmp(argv[n], "-mrtg") == 0) {
		    parameters->mrtg = 1;
		  }
		else
			err = 1;

		n++;
	}

	if (err)
		parameters->help = 1;
}

